	list      p=16f877            ; list directive to define processor
	#include <p16f877.inc>        ; processor specific variable definitions

	__CONFIG _CP_OFF & _WDT_ON & _BODEN_ON & _PWRTE_ON & _RC_OSC & _WRT_ENABLE_ON & _LVP_ON & _DEBUG_OFF & _CPD_OFF

LCD_DATA        EQU	PORTB           ; LCD data lines interface
LCD_DATA_TRIS   EQU	TRISB
LCD_CTRL        EQU	PORTA           ; LCD control lines interface

LCD_TEMP        EQU	0x020           ; LCD subroutines internal use

DELAY           EQU	0x023           ; Used in DELAYxxx routines
X_DELAY         EQU	0x024           ; Used in X_DELAYxxx routines

; PORTA control bits
LCD_E           EQU	2               ; LCD Enable control line
LCD_RW          EQU	1               ; LCD Read/Write control line
LCD_RS          EQU	0               ; LCD Register-Select control line


	ORG     0x000					; processor reset vector
		clrf    PCLATH				; ensure page bits are cleared
		goto    main				; go to beginning of program
;********************
	ORG		0x008

LCDINIT                             ; Busy-flag is not yet valid
	CLRF        LCD_CTRL            ; ALL PORT output should output Low.
									; power-up delay
	MOVLW       0x01E
	CALL        X_DELAY500          ; 30 * 0.5mS = 15mS
									; Busy Flag should be valid from here
	MOVLW       0x038               ; 8-bit-interface, 2-lines
	CALL        LCDPUTCMD
	MOVLW       0x000               ; disp.off, curs.off, no-blink
	CALL        LCDDMODE
	CALL        LCDCLEAR
	MOVLW       0x004               ; disp.on, curs.off
	CALL        LCDDMODE
	MOVLW       0x002               ; auto-inc (shift-cursor)
	CALL        LCDEMODE
	RETURN
;***********************************
LCDBUSY
	BSF         STATUS,RP0          ; Select Register page 1
	MOVLW       0x0FF               ; Set PORTB for input
	MOVWF       LCD_DATA_TRIS
	BCF         STATUS, RP0         ; Select Register page 0
	BCF         LCD_CTRL, LCD_RS    ; Set LCD for command mode
	BSF         LCD_CTRL, LCD_RW    ; Setup to read busy flag
	BSF         LCD_CTRL, LCD_E     ; LCD E-line High
	MOVF        LCD_DATA, W         ; Read busy flag + DDram address
	BCF         LCD_CTRL, LCD_E     ; LCD E-line Low
	ANDLW       0x80                ; Check Busy flag, High = Busy
	BTFSS       STATUS, Z
	GOTO        LCDBUSY
LCDNOTBUSY
	BCF         LCD_CTRL, LCD_RW
	BSF         STATUS, RP0         ; Select Register page 1
	MOVLW       0x000
	MOVWF       LCD_DATA_TRIS       ; Set PORTB for output
	BCF         STATUS, RP0         ; Select Register page 0
	RETURN
;***********************************
LCDCLEAR
	MOVLW       0x001
	CALL        LCDPUTCMD
	RETURN
;***********************************
LCDHOME
	MOVLW       0x002
	CALL        LCDPUTCMD
	RETURN
;***********************************
;- Sets entry mode of the LCD
;- Required entry mode must be set in W
;  b0    : 0 = no display shift, 1 = display shift
;  b1    : 0 = auto-decrement, 1 = auto-increment
;  b2-b7 : don't care
;***********************************
LCDEMODE
	ANDLW       0x003               ; Strip upper bits
	IORLW       0x004               ; Function set
	CALL        LCDPUTCMD
	RETURN
;***********************************
;- Sets display control
;- Required entry mode must be set in W
;  b0    : 0 = cursor blink off, 1 = cursor blink on (if b1 = 1)
;  b1    : 0 = cursor off,  1 = cursor on
;  b2    : 0 = display off, 1 = display on (display data remains in DD-RAM)
;  b3-b7 : don't care
;***********************************
LCDDMODE
	ANDLW       0x007               ; Strip upper bits
	IORLW       0x008               ; Function set
	CALL        LCDPUTCMD
	RETURN
;***********************************
;- Sets the Character-Generator-RAM address.
;  CGRAM data is read/written after this setting.
;- Required CGRAM address must be set in W
;   b0-5	: required CGRAM address
;   b6-7	: don't care
;***********************************
LCDSCGA
	ANDLW       0x03F               ; Strip upper bits
	IORLW       0x040               ; Function set
	CALL        LCDPUTCMD
	RETURN
;***********************************
;- Sets the Display-Data-RAM address.
;  DDRAM data is read/written after this setting.
;- Required entry mode must be set in W
;   b0-6	: required DDRAM address
;   b7	: don't care
;***********************************
LCDSDDA
	IORLW       0x080               ; Function set
	CALL        LCDPUTCMD
	RETURN
;***********************************
;- Returns address counter contents,
;  used for both DDRAM and CGRAM.
;- RAM address is returned in W
;***********************************
LCDGADDR
	BSF         STATUS,RP0          ; Select Register page 1
	MOVLW       0x0FF               ; Set PORTB for input
	MOVWF       LCD_DATA_TRIS
	BCF         STATUS, RP0         ; Select Register page 0
	BCF         LCD_CTRL, LCD_RS    ; Set LCD for command mode
	BSF         LCD_CTRL, LCD_RW    ; Setup to read busy flag
	BSF         LCD_CTRL, LCD_E     ; LCD E-line High
	MOVF        LCD_DATA, W         ; Read busy flag + RAM address
	BCF         LCD_CTRL, LCD_E     ; LCD E-line Low
	ANDLW       0x07F               ; Strip upper bit
	BCF         LCD_CTRL, LCD_RW
	BSF         STATUS, RP0         ; Select Register page 1
	MOVLW       0x000
	MOVWF       LCD_DATA_TRIS       ; Set PORTB for output
	BCF         STATUS, RP0         ; Select Register page 0
	RETURN
;***********************************
;- Sends character to LCD
;- Required character must be in W
;***********************************
LCDPUTCHAR
	MOVWF       LCD_TEMP            ; Character to send is in W
	CALL        LCDBUSY             ; Wait for LCD to be ready
	BCF         LCD_CTRL, LCD_RW    ; Set LCD in read mode
	BSF         LCD_CTRL, LCD_RS    ; Set LCD in data mode
	BSF         LCD_CTRL, LCD_E     ; LCD E-line High
	MOVF        LCD_TEMP, W
	MOVWF       LCD_DATA            ; Send data to LCD
	BCF         LCD_CTRL, LCD_E     ; LCD E-line Low
	RETURN
;***********************************
;- Sends command to LCD
;- Required command must be in W
;***********************************
LCDPUTCMD
	MOVWF       LCD_TEMP            ; Command to send is in W
	CALL        LCDBUSY             ; Wait for LCD to be ready
	BCF         LCD_CTRL, LCD_RW    ; Set LCD in read mode
	BCF         LCD_CTRL, LCD_RS    ; Set LCD in command mode
	BSF         LCD_CTRL, LCD_E     ; LCD E-line High
	MOVF        LCD_TEMP, W
	MOVWF       LCD_DATA            ; Send data to LCD
	BCF         LCD_CTRL, LCD_E     ; LCD E-line Low
	RETURN
;***********************************
;- Delays Used in LCDINIT subroutine
;- Required delay factor must be in W
;  (Could be coded more efficient,
;  but this approach gives more flexibility)
;***********************************  a 500uS delay @ 4MHz X-tal
DELAY500
	MOVLW       D'165'              ; +1        1 cycle
	MOVWF       DELAY               ; +2        1 cycle
DELAY500_LOOP
	DECFSZ      DELAY, F            ; step1     1 cycle
	GOTO        DELAY500_LOOP       ; step2     2 cycles
DELAY500_END
	RETURN                          ; +3        2 cycles
;***********************************  a delay of 'W' * 500mS
X_DELAY500
	MOVWF       X_DELAY             ; +1        1 cycle
X_DELAY500_LOOP
	CALL        DELAY500            ; step1     wait 500uSec
	DECFSZ      X_DELAY, F          ; step2     1 cycle
	GOTO        X_DELAY500_LOOP     ; step3     2 cycles
X_DELAY500_END
	RETURN                          ; +2        2 cycles
;***********************************


